/**
 * Copyright (c) 2013-2015 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry_overlay.core;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.IOException;
import java.math.BigInteger;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Properties;

import org.junit.BeforeClass;
import org.junit.Test;
import org.ow2.petals.registry_overlay.api.Constants;
import org.ow2.petals.registry_overlay.configuration.generated.Credentials;
import org.ow2.petals.registry_overlay.configuration.generated.Member;
import org.ow2.petals.registry_overlay.configuration.generated.Members;
import org.ow2.petals.registry_overlay.configuration.generated.PetalsRegistryOverlay;
import org.ow2.petals.registry_overlay.core.exception.ConfigurationException;
import org.ow2.petals.registry_overlay.core.exception.MemberIdentifierMissingException;
import org.ow2.petals.registry_overlay.core.exception.MissingCredentialsException;
import org.ow2.petals.registry_overlay.core.exception.MissingCredentialsGroupException;
import org.ow2.petals.registry_overlay.core.exception.MissingCredentialsPasswordException;
import org.ow2.petals.registry_overlay.core.exception.MissingMemberIdException;
import org.ow2.petals.registry_overlay.core.exception.MissingMembersException;
import org.ow2.petals.registry_overlay.core.exception.NotAClusterMemberException;
import org.ow2.petals.registry_overlay.core.exception.SeveralMemberIdException;

import com.ebmwebsourcing.easycommons.lang.reflect.ReflectionHelper;
import com.hazelcast.client.HazelcastClient;
import com.hazelcast.client.config.ClientConfig;
import com.hazelcast.client.config.ClientNetworkConfig;
import com.hazelcast.config.Config;
import com.hazelcast.config.ManagementCenterConfig;
import com.hazelcast.config.NetworkConfig;
import com.hazelcast.config.TcpIpConfig;
import com.hazelcast.core.HazelcastInstance;

/**
 * Unit tests of {@link RegistryOverlayNode}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class RegistryOverlayNodeTest {

    @BeforeClass
    public static void setup() throws URISyntaxException {
        final URL jdkLogCfgFileUrl = Thread.currentThread().getContextClassLoader()
                .getResource("logging.properties");
        System.setProperty("java.util.logging.config.file",
                new File(jdkLogCfgFileUrl.toURI()).getAbsolutePath());
    }

    /**
     * <p>
     * Test the creation and start of a Petals Registry Overlay node from a very basic configuration:
     * <ul>
     * <li>only one member,</li>
     * <li>the TCP port is not set,</li>
     * <li>in member properties, only the member id is set.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the default TCP port is used,</li>
     * <li>the following properties are set to their default values:
     * <ul>
     * <li>client map lock wait time</li>
     * </ul>
     * </li>
     * </ul>
     * </p>
     */
    @Test
    public void oneNode_01() throws ConfigurationException {

        // Create a registry with one member
        final String hostname = "localhost";
        final PetalsRegistryOverlay petalsRegistryOverlay = ClusterBuilder.createVeryBasicClusterConfig(hostname);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);
        final RegistryOverlayNode node = new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
        try {
            node.start();

            final HazelcastInstance hzlInstance = (HazelcastInstance) ReflectionHelper.getPrivateFieldValue(
                    RegistryOverlayNode.class, node, "instance");
            assertNotNull("Unexpected null Hazelcast instance", hzlInstance);
            final Config hzlConfig = hzlInstance.getConfig();
            assertNotNull("Unexpectedt null Hazelcast config", hzlConfig);
            final NetworkConfig hzlNetworkConfig = hzlConfig.getNetworkConfig();
            assertNotNull("Unexpected null Hazelcast network config", hzlNetworkConfig);
            assertNotNull("Unexpected null", hzlNetworkConfig.getJoin());
            final TcpIpConfig hzlTcpConfig = hzlNetworkConfig.getJoin().getTcpIpConfig();
            assertNotNull("Unexpected null Hazelcast Tcp config", hzlTcpConfig);
            assertTrue("Tcp not enable", hzlTcpConfig.isEnabled());
            assertNotNull("Tcp member list is null", hzlTcpConfig.getMembers());
            assertEquals("Unexpected Tcp member number", 0, hzlTcpConfig.getMembers().size());
            assertEquals("Unexpected Tcp listening port", Constants.DEFAULT_PORT, hzlNetworkConfig.getPort());
            final ManagementCenterConfig hzlMngtCenterConfig = hzlConfig.getManagementCenterConfig();
            assertNotNull("Management center is null", hzlMngtCenterConfig);
            assertTrue("Management center not enable", hzlMngtCenterConfig.isEnabled());
            assertEquals("Unexpected management center URL", RegistryOverlayNode.DEFAULT_MNGT_CENTER_URL,
                    hzlMngtCenterConfig.getUrl());

            final long clientMapLockWaitTime = (Long) ReflectionHelper.getPrivateFieldValue(RegistryOverlayNode.class,
                    node, "clientsMapLockWaitTime");
            assertEquals(RegistryOverlayNode.DEFAULT_CLIENTSMAP_LOCK_WAITTIME, clientMapLockWaitTime);

            // Connect to the registry
            final ClientConfig cltCfg = new ClientConfig();
            cltCfg.getGroupConfig().setName(ClusterBuilder.CREDENTIALS_GROUP);
            cltCfg.getGroupConfig().setPassword(ClusterBuilder.CREDENTIALS_PWD);
            final ClientNetworkConfig networkConfig = new ClientNetworkConfig();
            networkConfig.addAddress(hostname + ":" + Constants.DEFAULT_PORT);
            cltCfg.setNetworkConfig(networkConfig);
            final HazelcastInstance client = HazelcastClient.newHazelcastClient(cltCfg);
            client.shutdown();

        } finally {
            node.stop();
        }
    }

    /**
     * <p>
     * Test the creation of a Petals Registry Overlay node from a very basic configuration:
     * <ul>
     * <li>only one member,</li>
     * <li>the TCP port is not set,</li>
     * <li>in member properties:
     * <ul>
     * <li>the member id is set.</li>
     * <li>the client map lock wait time is set to an invalid value.</li>
     * </ul>
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the default TCP port is used,</li>
     * <li>the following properties are set to their default values:
     * <ul>
     * <li>client map lock wait time</li>
     * </ul>
     * </li>
     * </ul>
     * </p>
     */
    @Test
    public void warning_Configuration_InvalidPropertiesSetToTheirDefaultValues() throws ConfigurationException {

        // Create a registry with one member
        final String hostname = "localhost";
        final PetalsRegistryOverlay petalsRegistryOverlay = ClusterBuilder.createVeryBasicClusterConfig(hostname);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);
        memberProperties
.setProperty(Constants.MEMBER_PROPERTY_NAME_CLIENTSMAP_LOCK_WAITTIME, "invalid-value");
        final RegistryOverlayNode node = new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);

        final long clientMapLockWaitTime = (Long) ReflectionHelper.getPrivateFieldValue(RegistryOverlayNode.class,
                node, "clientsMapLockWaitTime");
        assertEquals(RegistryOverlayNode.DEFAULT_CLIENTSMAP_LOCK_WAITTIME, clientMapLockWaitTime);

    }

    /**
     * <p>
     * Test the creation of a Petals Registry Overlay node from a very basic configuration:
     * <ul>
     * <li>only one member,</li>
     * <li>the TCP port is not set,</li>
     * <li>in member properties:
     * <ul>
     * <li>the member id is set.</li>
     * <li>the client map lock wait time is set to an empty value.</li>
     * </ul>
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the default TCP port is used,</li>
     * <li>the following properties are set to their default values:
     * <ul>
     * <li>client map lock wait time</li>
     * </ul>
     * </li>
     * </ul>
     * </p>
     */
    @Test
    public void warning_Configuration_EmptyPropertiesSetToTheirDefaultValues() throws ConfigurationException {

        // Create a registry with one member
        final String hostname = "localhost";
        final PetalsRegistryOverlay petalsRegistryOverlay = ClusterBuilder.createVeryBasicClusterConfig(hostname);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_CLIENTSMAP_LOCK_WAITTIME, "");
        final RegistryOverlayNode node = new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);

        final long clientMapLockWaitTime = (Long) ReflectionHelper.getPrivateFieldValue(RegistryOverlayNode.class,
                node, "clientsMapLockWaitTime");
        assertEquals(RegistryOverlayNode.DEFAULT_CLIENTSMAP_LOCK_WAITTIME, clientMapLockWaitTime);

    }

    /**
     * <p>
     * Test the creation of a Petals Registry Overlay node from a very basic configuration:
     * <ul>
     * <li>only one member,</li>
     * <li>the TCP port is not set,</li>
     * <li>in member properties:
     * <ul>
     * <li>the member id is set.</li>
     * <li>the client map lock wait time is set to a valid value.</li>
     * </ul>
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the default TCP port is used,</li>
     * <li>the following properties are set to their expected values:
     * <ul>
     * <li>client map lock wait time</li>
     * </ul>
     * </li>
     * </ul>
     * </p>
     */
    @Test
    public void configuration_PropertiesSetToTheirExepectedValue() throws ConfigurationException {

        // Create a registry with one member
        final String hostname = "localhost";
        final PetalsRegistryOverlay petalsRegistryOverlay = ClusterBuilder.createVeryBasicClusterConfig(hostname);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);
        final String expectedClientMapLockWaitTime = "123";
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_CLIENTSMAP_LOCK_WAITTIME,
                expectedClientMapLockWaitTime);
        final RegistryOverlayNode node = new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);

        final long clientMapLockWaitTime = (Long) ReflectionHelper.getPrivateFieldValue(RegistryOverlayNode.class,
                node, "clientsMapLockWaitTime");
        assertEquals(Long.parseLong(expectedClientMapLockWaitTime), clientMapLockWaitTime);
    }

    /**
     * <p>
     * Test the creation and start of a Petals Registry Overlay node from a basic configuration
     * <ul>
     * <li>only one member,</li>
     * <li>the TCP port is set to the default value,</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error,</li>
     * <li>the default TCP port is used,</li>
     * </ul>
     * </p>
     */
    @Test
    public void oneNode_00() throws IOException, ConfigurationException {

        // Create a registry with one member
        final Member member = new Member();
        member.setId(ClusterBuilder.MEMBER_ID);
        member.setValue("localhost");
        member.setPort(BigInteger.valueOf(Constants.DEFAULT_PORT));
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        members.getMember().add(member);
        final Credentials credentials = new Credentials();
        credentials.setGroup(ClusterBuilder.CREDENTIALS_GROUP);
        credentials.setPassword(ClusterBuilder.CREDENTIALS_PWD);
        petalsRegistryOverlay.setCredentials(credentials);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);
        final RegistryOverlayNode node = new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
        node.start();

        try {
            final HazelcastInstance hzlInstance = (HazelcastInstance) ReflectionHelper.getPrivateFieldValue(
                    RegistryOverlayNode.class, node, "instance");
            assertNotNull("Unexpected null Hazelcast instance", hzlInstance);
            final Config hzlConfig = hzlInstance.getConfig();
            assertNotNull("Unexpectedt null Hazelcast config", hzlConfig);
            final NetworkConfig hzlNetworkConfig = hzlConfig.getNetworkConfig();
            assertNotNull("Unexpected null Hazelcast network config", hzlNetworkConfig);
            assertNotNull("Unexpected null", hzlNetworkConfig.getJoin());
            final TcpIpConfig hzlTcpConfig = hzlNetworkConfig.getJoin().getTcpIpConfig();
            assertNotNull("Unexpected null Hazelcast Tcp config", hzlTcpConfig);
            assertTrue("Tcp not enable", hzlTcpConfig.isEnabled());
            assertNotNull("Tcp member list is null", hzlTcpConfig.getMembers());
            assertEquals("Unexpected Tcp member number", 0, hzlTcpConfig.getMembers().size());
            assertEquals("Unexpected Tcp listening port", Constants.DEFAULT_PORT, hzlNetworkConfig.getPort());
            final ManagementCenterConfig hzlMngtCenterConfig = hzlConfig.getManagementCenterConfig();
            assertNotNull("Management center is null", hzlMngtCenterConfig);
            assertTrue("Management center not enable", hzlMngtCenterConfig.isEnabled());
            assertEquals("Unexpected management center URL", RegistryOverlayNode.DEFAULT_MNGT_CENTER_URL,
                    hzlMngtCenterConfig.getUrl());

            // Connect to the registry
            final ClientConfig cltCfg = new ClientConfig();
            cltCfg.getGroupConfig().setName(ClusterBuilder.CREDENTIALS_GROUP);
            cltCfg.getGroupConfig().setPassword(ClusterBuilder.CREDENTIALS_PWD);
            final ClientNetworkConfig networkConfig = new ClientNetworkConfig();
            networkConfig.addAddress(member.getValue() + ":" + member.getPort().toString());
            cltCfg.setNetworkConfig(networkConfig);
            final HazelcastInstance client = HazelcastClient.newHazelcastClient(cltCfg);
            client.shutdown();
        } finally {
            node.stop();
        }
    }

    /**
     * <p>
     * Check the constructor {@link RegistryOverlayNode#RegistryOverlayNode(PetalsRegistryOverlay, Properties)} against
     * the Petals registry overlay configuration defined:
     * <ul>
     * <li>a basic configuration,</li>
     * <li>without credentials.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about missing credentials.
     * </p>
     */
    @Test(expected = MissingCredentialsException.class)
    public void error_ConfigurationWithoutCredentials() throws ConfigurationException {

        // Create the petals registry overlay configuration
        final Member member = new Member();
        member.setId(ClusterBuilder.MEMBER_ID);
        member.setValue("localhost");
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        members.getMember().add(member);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);

        new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
    }

    /**
     * <p>
     * Check the constructor {@link RegistryOverlayNode#RegistryOverlayNode(PetalsRegistryOverlay, Properties)} against
     * the Petals registry overlay configuration defined:
     * <ul>
     * <li>a basic configuration,</li>
     * <li>with credentials where the group is missing.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about missing credentials group.
     * </p>
     */
    @Test(expected = MissingCredentialsGroupException.class)
    public void error_ConfigurationWithoutGroup() throws ConfigurationException {

        // Create the petals registry overlay configuration
        final Member member = new Member();
        member.setId(ClusterBuilder.MEMBER_ID);
        member.setValue("localhost");
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        members.getMember().add(member);
        final Credentials credentials = new Credentials();
        credentials.setPassword(ClusterBuilder.CREDENTIALS_PWD);
        petalsRegistryOverlay.setCredentials(credentials);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);

        new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
    }

    /**
     * <p>
     * Check the constructor {@link RegistryOverlayNode#RegistryOverlayNode(PetalsRegistryOverlay, Properties)} against
     * the Petals registry overlay configuration defined:
     * <ul>
     * <li>a basic configuration,</li>
     * <li>with credentials where the group is empty.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about missing credentials group.
     * </p>
     */
    @Test(expected = MissingCredentialsGroupException.class)
    public void error_ConfigurationWithEmptyGroup() throws ConfigurationException {

        // Create the petals registry overlay configuration
        final Member member = new Member();
        member.setId(ClusterBuilder.MEMBER_ID);
        member.setValue("localhost");
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        members.getMember().add(member);
        final Credentials credentials = new Credentials();
        credentials.setGroup("");
        credentials.setPassword(ClusterBuilder.CREDENTIALS_PWD);
        petalsRegistryOverlay.setCredentials(credentials);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);

        new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
    }

    /**
     * <p>
     * Check the constructor {@link RegistryOverlayNode#RegistryOverlayNode(PetalsRegistryOverlay, Properties)} against
     * the Petals registry overlay configuration defined:
     * <ul>
     * <li>a basic configuration,</li>
     * <li>with credentials where the password is missing.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about missing credentials password.
     * </p>
     */
    @Test(expected = MissingCredentialsPasswordException.class)
    public void error_ConfigurationWithoutPassword() throws ConfigurationException {

        // Create the petals registry overlay configuration
        final Member member = new Member();
        member.setId(ClusterBuilder.MEMBER_ID);
        member.setValue("localhost");
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        members.getMember().add(member);
        final Credentials credentials = new Credentials();
        credentials.setGroup(ClusterBuilder.CREDENTIALS_GROUP);
        petalsRegistryOverlay.setCredentials(credentials);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);

        new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
    }

    /**
     * <p>
     * Check the constructor {@link RegistryOverlayNode#RegistryOverlayNode(PetalsRegistryOverlay, Properties)} against
     * the Petals registry overlay configuration defined:
     * <ul>
     * <li>a basic configuration,</li>
     * <li>with credentials where the password is empty.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about missing credentials password.
     * </p>
     */
    @Test(expected = MissingCredentialsPasswordException.class)
    public void error_ConfigurationWithEmptyPassword() throws ConfigurationException {

        // Create the petals registry overlay configuration
        final Member member = new Member();
        member.setId(ClusterBuilder.MEMBER_ID);
        member.setValue("localhost");
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        members.getMember().add(member);
        final Credentials credentials = new Credentials();
        credentials.setGroup(ClusterBuilder.CREDENTIALS_GROUP);
        credentials.setPassword("");
        petalsRegistryOverlay.setCredentials(credentials);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);

        new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
    }

    /**
     * <p>
     * Check the constructor {@link RegistryOverlayNode#RegistryOverlayNode(PetalsRegistryOverlay, Properties)} against
     * the Petals registry overlay configuration defined:
     * <ul>
     * <li>a basic configuration,</li>
     * <li>without members</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about missing member.
     * </p>
     */
    @Test(expected = MissingMembersException.class)
    public void error_ConfigurationWithoutMember() throws ConfigurationException {

        // Create the petals registry overlay configuration
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Credentials credentials = new Credentials();
        credentials.setGroup(ClusterBuilder.CREDENTIALS_GROUP);
        credentials.setPassword(ClusterBuilder.CREDENTIALS_PWD);
        petalsRegistryOverlay.setCredentials(credentials);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);

        new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
    }

    /**
     * <p>
     * Check the constructor {@link RegistryOverlayNode#RegistryOverlayNode(PetalsRegistryOverlay, Properties)} against
     * the Petals registry overlay configuration defined:
     * <ul>
     * <li>a basic configuration,</li>
     * <li>with an empty member list</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about missing member.
     * </p>
     */
    @Test(expected = MissingMembersException.class)
    public void error_ConfigurationWithEmptyMemberList() throws ConfigurationException {

        // Create the petals registry overlay configuration
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Credentials credentials = new Credentials();
        credentials.setGroup(ClusterBuilder.CREDENTIALS_GROUP);
        credentials.setPassword(ClusterBuilder.CREDENTIALS_PWD);
        petalsRegistryOverlay.setCredentials(credentials);
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);

        new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
    }

    /**
     * <p>
     * Check the constructor {@link RegistryOverlayNode#RegistryOverlayNode(PetalsRegistryOverlay, Properties)} against
     * the Petals registry overlay configuration defined:
     * <ul>
     * <li>a basic configuration,</li>
     * <li>with a member configuration where the identifier is missing</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about missing member.
     * </p>
     */
    @Test(expected = MissingMemberIdException.class)
    public void error_ConfigurationWithMemberWithoutId() throws ConfigurationException {

        // Create the petals registry overlay configuration
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Credentials credentials = new Credentials();
        credentials.setGroup(ClusterBuilder.CREDENTIALS_GROUP);
        credentials.setPassword(ClusterBuilder.CREDENTIALS_PWD);
        petalsRegistryOverlay.setCredentials(credentials);
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        final Member member = new Member();
        member.setValue("localhost");
        members.getMember().add(member);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);

        new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
    }

    /**
     * <p>
     * Check the constructor {@link RegistryOverlayNode#RegistryOverlayNode(PetalsRegistryOverlay, Properties)} against
     * the Petals registry overlay configuration defined:
     * <ul>
     * <li>a basic configuration,</li>
     * <li>with a member configuration where the identifier is empty</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about missing member.
     * </p>
     */
    @Test(expected = MissingMemberIdException.class)
    public void error_ConfigurationWithMemberWithEmptyId() throws ConfigurationException {

        // Create the petals registry overlay configuration
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Credentials credentials = new Credentials();
        credentials.setGroup(ClusterBuilder.CREDENTIALS_GROUP);
        credentials.setPassword(ClusterBuilder.CREDENTIALS_PWD);
        petalsRegistryOverlay.setCredentials(credentials);
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        final Member member = new Member();
        member.setId("");
        member.setValue("localhost");
        members.getMember().add(member);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);

        new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
    }

    /**
     * <p>
     * Check the constructor {@link RegistryOverlayNode#RegistryOverlayNode(PetalsRegistryOverlay, Properties)} against
     * the Petals registry overlay configuration defined:
     * <ul>
     * <li>a basic configuration,</li>
     * <li>with an empty member identifier in the local member configuration.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about missing member in the local member configuration.
     * </p>
     */
    @Test(expected = MemberIdentifierMissingException.class)
    public void error_ConfigurationWithEmptyMemberInLocalConfig() throws ConfigurationException {

        // Create the petals registry overlay configuration
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Credentials credentials = new Credentials();
        credentials.setGroup(ClusterBuilder.CREDENTIALS_GROUP);
        credentials.setPassword(ClusterBuilder.CREDENTIALS_PWD);
        petalsRegistryOverlay.setCredentials(credentials);
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        final Member member = new Member();
        member.setId(ClusterBuilder.MEMBER_ID);
        member.setValue("localhost");
        members.getMember().add(member);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, "");

        new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
    }

    /**
     * <p>
     * Check the constructor {@link RegistryOverlayNode#RegistryOverlayNode(PetalsRegistryOverlay, Properties)} against
     * the Petals registry overlay configuration defined:
     * <ul>
     * <li>a basic configuration,</li>
     * <li>with a missing member identifier in the local member configuration.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about missing member in the local configuration member.
     * </p>
     */
    @Test(expected = MemberIdentifierMissingException.class)
    public void error_ConfigurationWithMissingMemberInLocalConfig()
            throws ConfigurationException {

        // Create the petals registry overlay configuration
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Credentials credentials = new Credentials();
        credentials.setGroup(ClusterBuilder.CREDENTIALS_GROUP);
        credentials.setPassword(ClusterBuilder.CREDENTIALS_PWD);
        petalsRegistryOverlay.setCredentials(credentials);
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        final Member member = new Member();
        member.setId(ClusterBuilder.MEMBER_ID);
        member.setValue("localhost");
        members.getMember().add(member);
        final Properties memberProperties = new Properties();

        new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
    }

    /**
     * <p>
     * Check the constructor {@link RegistryOverlayNode#RegistryOverlayNode(PetalsRegistryOverlay, Properties)} against
     * the Petals registry overlay configuration defined:
     * <ul>
     * <li>a basic configuration,</li>
     * <li>with an unknown member identifier in the local member configuration.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the member identifier not found in the cluster configuration.
     * </p>
     */
    @Test(expected = NotAClusterMemberException.class)
    public void error_ConfigurationWithUnknownMemberInLocalConfig()
            throws ConfigurationException {

        // Create the petals registry overlay configuration
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Credentials credentials = new Credentials();
        credentials.setGroup(ClusterBuilder.CREDENTIALS_GROUP);
        credentials.setPassword(ClusterBuilder.CREDENTIALS_PWD);
        petalsRegistryOverlay.setCredentials(credentials);
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        final Member member = new Member();
        member.setId(ClusterBuilder.MEMBER_ID);
        member.setValue("localhost");
        members.getMember().add(member);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, "unknown-id");

        new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
    }

    /**
     * <p>
     * Check the constructor {@link RegistryOverlayNode#RegistryOverlayNode(PetalsRegistryOverlay, Properties)} against
     * the Petals registry overlay configuration defined:
     * <ul>
     * <li>a basic configuration,</li>
     * <li>with several member with the same identifier in the cluster configuration.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: An error occurs about the members with the same identifier.
     * </p>
     */
    @Test(expected = SeveralMemberIdException.class)
    public void error_ConfigurationWithSeveralMemberId() throws ConfigurationException {

        // Create the petals registry overlay configuration
        final PetalsRegistryOverlay petalsRegistryOverlay = new PetalsRegistryOverlay();
        final Credentials credentials = new Credentials();
        credentials.setGroup(ClusterBuilder.CREDENTIALS_GROUP);
        credentials.setPassword(ClusterBuilder.CREDENTIALS_PWD);
        petalsRegistryOverlay.setCredentials(credentials);
        final Members members = new Members();
        petalsRegistryOverlay.setMembers(members);
        final Member member1 = new Member();
        member1.setId(ClusterBuilder.MEMBER_ID);
        member1.setValue("localhost");
        members.getMember().add(member1);
        final Member member2 = new Member();
        member2.setId(ClusterBuilder.MEMBER_ID);
        member2.setValue("localhost");
        members.getMember().add(member2);
        final Properties memberProperties = new Properties();
        memberProperties.setProperty(Constants.MEMBER_PROPERTY_NAME_MEMBER_ID, ClusterBuilder.MEMBER_ID);

        new RegistryOverlayNode(petalsRegistryOverlay, memberProperties);
    }
}
